/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.geoviewer;

import java.awt.Color;
import java.util.Arrays;
import java.util.Objects;
import nl.cloudfarming.client.util.UOM;

/**
 * Palette for Legends
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public abstract class LegendPalette<E> extends Palette<E> {

    public static final String PROP_LEGEND_COLORS = "legend_colors";
    
    // TODO: make updatable, LegendPanel will have to listen for uom changes
    private final UOM uom;

    public LegendPalette(Color baseColor, UOM uom) {
        super(baseColor);
        this.uom = uom;
    }

    public UOM getUom() {
        return uom;
    }

    public abstract LegendColors[] getColors();

    public abstract class LegendColors {

        public abstract Color getColor();

        public abstract String getDescription();

        @Override
        public final boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final LegendColors other = (LegendColors) obj;
            if (this.getColor() != null && other.getColor() != null) {
                if (!this.getColor().equals(other.getColor())) {
                    return false;
                }
            } else if (!(this.getColor() == null && other.getColor() == null)) {
                return false;
            }
            if (!this.getDescription().equals(other.getDescription())) {
                return false;
            }
            return true;
        }

        @Override
        public int hashCode() {
            int hash = 5;
            hash = 17 * hash + (this.getColor() != null ? this.getColor().hashCode() : 0);
            hash = 17 * hash + (this.getDescription() != null ? this.getDescription().hashCode() : 0);
            return hash;
        }
    }

    @Override
    public final boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final LegendPalette other = (LegendPalette) obj;
        if (!Objects.equals(this.uom, other.uom)) {
            return false;
        }
        if (!Arrays.equals(this.getColors(), other.getColors())) {
            return false;
        }
        return true;
    }

    @Override
    public final int hashCode() {
        int hash = 5;
        int colorHash = 0;
        if (this.getColors() != null) {
            for (LegendColors legensColor : this.getColors()) {
                colorHash += legensColor.hashCode();
            }
        }
        hash = 17 * hash + colorHash;
        return hash;
    }
}
